import string

import requests
from requests import Response
import urllib3
from urllib3.exceptions import InsecureRequestWarning

from typing import Tuple, Optional, List, Dict

from bs4 import BeautifulSoup
from bs4.element import Comment

from sklearn.feature_extraction.text import TfidfVectorizer  # type: ignore
from sklearn.feature_extraction import text as sklearn_text  # type: ignore

import nltk  # type: ignore
from nltk.corpus import stopwords  # type: ignore

urllib3.disable_warnings(InsecureRequestWarning)
nltk.download('punkt')
nltk.download('stopwords')


##############
# Similarity #
##############

def sk_similarity(doc1: str, doc2: str) -> float:
    ENGLISH_STOP_WORDS = set(stopwords.words('english')).union(set(sklearn_text.ENGLISH_STOP_WORDS)).union(set(string.punctuation))
    vectorizer = TfidfVectorizer(stop_words=list(ENGLISH_STOP_WORDS), max_features=5000)
    tfidf = vectorizer.fit_transform([doc1, doc2])

    return round(((tfidf * tfidf.T).toarray())[0, 1] * 100)


##################
# Web treatment #
#################

def tag_visible(element) -> bool:
    """Identified element present in specific balise"""
    if element.parent.name in ['style', 'script', 'head', 'title', 'meta', '[document]']:
        return False
    if isinstance(element, Comment):
        return False
    return True


def text_from_html(body: str) -> str:
    """Extract text from web page and remove text present in specific balise"""
    soup = BeautifulSoup(body, 'lxml')
    texts = soup.findAll(text=True)
    visible_texts = filter(tag_visible, texts)
    return u" \n".join(t.strip() for t in visible_texts)


def find_list_resources(tag: str, attribute: str, soup: BeautifulSoup) -> List[str]:
    """Find ressource in web page list in attribute"""
    list_resources = []
    for x in soup.findAll(tag):
        try:
            list_resources.append(x[attribute])
        except KeyError:
            pass
    return list_resources


def website_ressource(soup: BeautifulSoup) -> Dict[str, List[str]]:
    """Collect all website's ressources"""
    ressource_dict = dict()
    ressource_dict["image_scr"] = find_list_resources('img', "src", soup)
    ressource_dict["script_src"] = find_list_resources('script', "src", soup)
    ressource_dict["css_link"] = find_list_resources("link", "href", soup)
    ressource_dict["source_src"] = find_list_resources("source", "src", soup)
    ressource_dict["a_href"] = find_list_resources("a", "href", soup)
    ressource_dict["form_action"] = find_list_resources("form", "action", soup)

    return ressource_dict


def ressource_difference(original_ressource: Dict[str, List[str]], compare_ressource: Dict[str, List[str]]) -> str:
    """Difference between original website's ressources and comparaison website's ressources"""
    cp_total = 0
    cp_diff = 0

    for r_to_check in original_ressource:
        for r in original_ressource[r_to_check]:
            cp_total += 1
            if r_to_check in compare_ressource:
                if r not in compare_ressource[r_to_check]:
                    cp_diff += 1

    return str(int((cp_diff / cp_total) * 100))


def ratio(ressource_diff: str, similarity: str) -> float:
    """Calculate the possibility of similarity based on ressource difference en tfidf similarity"""
    if int(ressource_diff) != 0:
        if int(ressource_diff) < int(similarity):
            return round((int(ressource_diff) / int(similarity)) * int(ressource_diff), 2)
        else:
            return round((int(similarity) / int(ressource_diff)) * int(similarity), 2)
    elif int(similarity) == 100:
        return 0.05
    else:
        return 0.5


def get_website(website: str) -> Optional[Response]:
    """Request the website pass in parameter"""
    url = f"http://{website}"

    try:
        return requests.get(url, verify=False, timeout=3)
    except urllib3.exceptions.NewConnectionError:
        return None
    except requests.exceptions.ConnectionError:
        return None
    except urllib3.exceptions.ReadTimeoutError:
        return None
    except Exception:
        # import traceback
        # traceback.print_exception(type(e), e, e.__traceback__)
        return None


def extract_text_ressource(website_text: str) -> Tuple[str, Dict[str, List[str]]]:
    """Extract the text from html and collect all ressources"""
    soup = BeautifulSoup(website_text, "lxml")

    text = text_from_html(website_text)
    ressource = website_ressource(soup)

    return text, ressource



import argparse

from .similarius import sk_similarity
from .similarius import text_from_html  # noqa
from .similarius import find_list_resources  # noqa
from .similarius import ressource_difference
from .similarius import ratio
from .similarius import get_website
from .similarius import extract_text_ressource


def main():

    parser = argparse.ArgumentParser()
    parser.add_argument("-o", "--original", help="Website to compare")
    parser.add_argument("-w", "--website", nargs="+", help="Website to compare")
    args = parser.parse_args()

    # Original
    original = get_website(args.original)

    if not original:
        print("[-] The original website is unreachable...")
        exit(1)

    original_text, original_ressource = extract_text_ressource(original.text)

    for website in args.website:
        print(f"\n********** {args.original} <-> {website} **********")

        # Compare
        compare = get_website(website)

        if not compare:
            print(f"[-] {website} is unreachable...")
            continue

        compare_text, compare_ressource = extract_text_ressource(compare.text)

        # Calculate
        sim = str(sk_similarity(compare_text, original_text))
        print(f"\nSimilarity: {sim}")

        ressource_diff = ressource_difference(original_ressource, compare_ressource)
        print(f"Ressource Difference: {ressource_diff}")

        ratio_compare = ratio(ressource_diff, sim)
        print(f"Ratio: {ratio_compare}")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['similarius']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.11.1,<5.0.0',
 'lxml>=4.9.2,<5.0.0',
 'nltk>=3.8.1,<4.0.0',
 'requests>=2.28.1,<3.0.0',
 'scikit-learn>=1.2.0,<2.0.0']

entry_points = \
{'console_scripts': ['similarius = similarius:main']}

setup_kwargs = {
    'name': 'similarius',
    'version': '0.0.1',
    'description': 'Compare web page and evaluate the level of similarity.',
    'long_description': '# Similarius\n\nSimilarius is a Python library to compare web page and evaluate the level of similarity.\n\nThe tool can be used as a stand-alone tool or to feed other systems.\n\n\n\n# Requirements\n\n- Python 3.8+\n- [Requests](https://github.com/psf/requests)\n- [Scikit-learn](https://github.com/scikit-learn/scikit-learn)\n- [Beautifulsoup4](https://pypi.org/project/beautifulsoup4/)\n- [nltk](https://github.com/nltk/nltk)\n\n\n\n# Installation\n\n## Source install\n\n**Similarius** can be install with poetry. If you don\'t have poetry installed, you can do the following `curl -sSL https://raw.githubusercontent.com/python-poetry/poetry/master/get-poetry.py | python`.\n\n~~~bash\n$ poetry install\n$ poetry shell\n$ similarius -h\n~~~\n\n## pip installation\n\n~~~bash\n$ pip3 install similarius\n~~~\n\n\n\n# Usage\n\n~~~bash\ndacru@dacru:~/git/Similarius/similarius$ similarius --help\nusage: similarius.py [-h] [-o ORIGINAL] [-w WEBSITE [WEBSITE ...]]\n\noptional arguments:\n  -h, --help            show this help message and exit\n  -o ORIGINAL, --original ORIGINAL\n                        Website to compare\n  -w WEBSITE [WEBSITE ...], --website WEBSITE [WEBSITE ...]\n                        Website to compare\n~~~\n\n\n\n# Usage example\n\n~~~bash\ndacru@dacru:~/git/Similarius/similarius$ similarius -o circl.lu -w europa.eu circl.eu circl.lu\n~~~\n\n\n\n# Used as a library\n\n~~~python\nimport argparse\nfrom similarius import get_website, extract_text_ressource, sk_similarity, ressource_difference, ratio\n\nparser = argparse.ArgumentParser()\nparser.add_argument("-w", "--website", nargs="+", help="Website to compare")\nparser.add_argument("-o", "--original", help="Website to compare")\nargs = parser.parse_args()\n\n# Original\noriginal = get_website(args.original)\n\nif not original:\n    print("[-] The original website is unreachable...")\n    exit(1)\n\noriginal_text, original_ressource = extract_text_ressource(original.text)\n\nfor website in args.website:\n    print(f"\\n********** {args.original} <-> {website} **********")\n\n    # Compare\n    compare = get_website(website)\n\n    if not compare:\n        print(f"[-] {website} is unreachable...")\n        continue\n\n    compare_text, compare_ressource = extract_text_ressource(compare.text)\n\n    # Calculate\n    sim = str(sk_similarity(compare_text, original_text))\n    print(f"\\nSimilarity: {sim}")\n\n    ressource_diff = ressource_difference(original_ressource, compare_ressource)\n    print(f"Ressource Difference: {ressource_diff}")\n\n    ratio_compare = ratio(ressource_diff, sim)\n    print(f"Ratio: {ratio_compare}")\n~~~\n\n',
    'author': 'David Cruciani',
    'author_email': 'david.cruciani@securitymadein.lu',
    'maintainer': 'Alexandre Dulaunoy',
    'maintainer_email': 'a@foo.be',
    'url': 'https://github.com/ail-project/Similarius',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
